from random import *
from pygame.locals import *
import pygame
import time
import math
import sys,os

SCREENSIZE = [800,600]
pygame.font.init()

class Feather:
    def __init__(self,x):
        typeno = randint(0,20)
        self.x = x + 20
        self.y = 100
        types = {"Red"    : "feather.png",
                 "Orange" : "feather2.png",
                 "Yellow" : "feather3.png"}
        if typeno <= 12:
            self.type = "Red"
            self.rect = pygame.Rect(self.x,self.y,75,30)
        if typeno >= 13 and typeno <= 17:
            self.type = "Orange"
            self.rect = pygame.Rect(self.x,self.y,50,20)
        if typeno >=18:
            self.type = "Yellow"
            self.rect = pygame.Rect(self.x,self.y,35,15)
        self.base_image = pygame.image.load(types[self.type])
        self.in_cauldron_var = 0
    
    def in_cauldron(self):
        self.in_cauldron_var = 1
    
    def blit(self,screen):
        screen.blit(self.base_image,(self.x,self.y))
        
    def get_pos(self):
        return self.x,self.y
    
    def move(self,wind):
        self.x += 0 + wind[0]
        self.y += 0.5 + wind[1]
        if self.type == "Red":
            self.y += 3
            self.rect.move_ip(0 + wind[0], 2.5 + wind[1])
        elif self.type == "Orange":
            self.y += 1
            self.rect.move_ip(0 + wind[0], 1.5 + wind[1])
        elif self.type == "Yellow":
            self.y += 0.5
            self.rect.move_ip(0 + wind[0], 0.5 + wind[1])
        if self.y >= SCREENSIZE[1] - 20:
            self.y = SCREENSIZE[1] - 20
        if self.x <= 60:
            self.x = 60
        if self.x >= SCREENSIZE[0] - 60:
            self.x = SCREENSIZE[0] - 60

class Bird:
    def __init__(self):
        self.base_image = "parrot"
        self.image = []
        for i in range(1,13):
            image = pygame.image.load(self.base_image+str(i)+".png")
            self.image.append(image)
        self.x = 50
        self.y = 0
        self.i = 0
        self.wingdir = 0
        self.dir = 0
    
    def load_img(self):
        new_img_list = []
        for i in range(1,13):
            image = pygame.transform.flip(self.image[i-1],1,0)
            new_img_list.append(image)
        self.image = new_img_list
    
    def blit(self,screen):
        screen.blit(self.image[self.i],(self.x, self.y))
        if self.i == 11:
            self.wingdir = 1
        if self.i == 0:
            self.wingdir = 0
        
        if self.wingdir == 0:
            self.i += 1
        if self.wingdir == 1:
            self.i -= 1
    
    def move(self):
        number1 = randint(0,41)
        number2 = randint(0,41)
        if number1 == number2 and self.x <= SCREENSIZE[0]-170 and self.x >= 20:
            if self.dir == 0:
                self.dir = 1 - self.dir
                self.load_img()
        if self.x >= SCREENSIZE[0]-150 or self.x <= 0:
            self.dir = self.dir = 1 - self.dir
            self.load_img()
        
        if self.dir == 0:
            self.x += 10
        if self.dir == 1:
            self.x -= 10
    
    def drop_feather(self):
        return Feather(self.x)

class Level:
    #size, wind_position, airstream_size)
    definitions = {"fan-left"     : ([75,150],[40,00],75),
                   "fan-right"    : ([75,150],[35,00],75),
                   "window-right" : ([30,150],[0,100],100),
                   "window-left" : ([30,150],[30,100],100),
                   "pig-right"    : ([200,200],[160,50],50),
                   "pig-left"    : ([200,200],[40,50],50)}
    
    def __init__(self):
        #self.w1 = Windmachine(-1,5,"fan1.png",[20,300],[75,150])
        self.levelno = 1
    
    def load_level(self):
        self.f = open("level" + str(self.levelno) + ".dat")
        i = 0
        for line in self.f:
            i += 1
        self.f.close()
        self.f = open("level" + str(self.levelno) + ".dat")
        self.machines = []
        self.feathers_needed = self.f.readline().split(":")
        temp = []
        for j in range(0,len(self.feathers_needed)):
            temp.append(int(self.feathers_needed[j]))
        self.feathers_needed = []
        for j in range(0,len(temp)):
            self.feathers_needed.append(temp[j])
        for j in range(1,i):
            type,pos = self.f.readline().split(":")
            strength,direction,spread,pos = pos.split("!")
            x,y = pos.split(",")
            y = y.replace("\n","")
            pos = int(x),int(y)
            m = Windmachine(time.time(),
                            int(strength),
                            type,
                            pos,
                            self.definitions[type][0],
                            self.definitions[type][1],
                            self.definitions[type][2],
                            int(direction),
                            int(spread))
            self.machines.append(m)
        self.f.close()
        self.levelno += 1
        
    def blit(self,screen):
        for i in self.machines:
            i.blit(screen)

class Windmachine:
    def __init__(self,
                time,
                power,
                base_image,
                pos,
                size,
                airstream_pos,
                airstream_size,
                airstream_angle,
                airstream_spread):
        """time is how long you want it to stay on for, -1 for forever,
        power is how powerful it is on a rate of 1-5, size is the height of
        the base image, airstream_size is the width of the airstream, airstream
        angle is the direction of the airstream measured in degrees clockwise from
        vertically upwards"""
        self.time = time
        self.power = power
        self.base_image = base_image
        self.image = pygame.image.load(base_image+"-off.png")
        self.pos = pos
        self.size = size
        #if self.x < (SCREENSIZE[0]/2)-(self.size[0]/2):
        #    self.direction = 1
        #else:
        #    self.direction = 0
        self.speed = self.power*25
        self.width = 50
        self.spread = 50
        self.state = 0
        self.state_times = []
        self.rect = pygame.Rect(self.pos,self.size)
        
        self.airstream_size = airstream_size
        #Convert the angles into radians
        self.airstream_angle = airstream_angle * math.pi / 180
        self.airstream_spread = airstream_spread * math.pi / 180
        #airstream_angle_1 and _2 are the outer boundaries of the airstream
        #expressed in radians from the vertical
        self.airstream_angle_1 = self.airstream_angle - self.airstream_spread / 2
        self.airstream_angle_2 = self.airstream_angle + self.airstream_spread / 2
        #airstream_pos_x, _y are the central point of the airstream
        self.airstream_pos_x = airstream_pos[0] + pos[0]
        self.airstream_pos_y = airstream_pos[1] + pos[1]
        #airstream_pos_x_1, etc are the outer points of the airstream
        self.airstream_pos_1_x = self.airstream_pos_x - airstream_size * math.sin(0.5 * math.pi - self.airstream_angle) / 2
        self.airstream_pos_1_y = self.airstream_pos_y + airstream_size * math.cos(0.5 * math.pi - self.airstream_angle) / 2
        self.airstream_pos_2_x = self.airstream_pos_x + airstream_size * math.sin(0.5 * math.pi - self.airstream_angle) / 2
        self.airstream_pos_2_y = self.airstream_pos_y - airstream_size * math.cos(0.5 * math.pi - self.airstream_angle) / 2
    
    def blit(self,screen):
        screen.blit(self.image,self.pos)
    
    def change_state(self):
        self.state = 1 - self.state
        if self.state == 1:
            self.image = pygame.image.load(self.base_image+"-on.png")
        if self.state == 0:
            self.image = pygame.image.load(self.base_image+"-off.png")
        self.state_times.append([self.state,time.time()])
    
    def get_wind(self,wind_pos):
        
        local_power_x, local_power_y = (0, 0)
        if self.state == 1:
            wind_pos_x, wind_pos_y = wind_pos
            #Find the points at which the airstream intercepts the vertical
            #line drawn through wind_pos
            tangent = math.tan(self.airstream_angle_1) 
            if tangent != 0:
                intercept_1 = (wind_pos_x - self.airstream_pos_1_x) / tangent + self.airstream_pos_1_y
                distance_1 = (wind_pos_x - self.airstream_pos_1_x) / math.sin(self.airstream_angle_1)
            else:
                intercept_1 = None
                distance_1 = None
            
            tangent = math.tan(self.airstream_angle_2) 
            if tangent != 0:
                intercept_2 = (wind_pos_x - self.airstream_pos_2_x) / math.tan(self.airstream_angle_2) + self.airstream_pos_2_y
                distance_2 = (wind_pos_x - self.airstream_pos_2_x) / math.sin(self.airstream_angle_2)
            else:
                intercept_2 = None
                distance_2 = None
            
            #Various tests to see if we're in the windstream
            upper_limit = 9999999
            if distance_1 > 0 and math.sin(self.airstream_angle_1) > 0:
                upper_limit = intercept_1
            elif distance_2 > 0 and math.sin(self.airstream_angle_2) < 0:
                upper_limit = intercept_2
                
            lower_limit = 0
            if distance_1 > 0 and math.sin(self.airstream_angle_1) < 0:
                lower_limit = intercept_1
            elif distance_2 > 0 and math.sin(self.airstream_angle_2) > 0:
                lower_limit = intercept_2
                
            in_airstream = False
            if wind_pos_y < upper_limit and wind_pos_y > lower_limit:
                in_airstream = True
            if distance_1 < 0 and distance_2 is None:
                in_airstream = False
            if distance_2 < 0 and distance_1 is None:
                in_airstream = False
            
            distance = (wind_pos_x - self.airstream_pos_x) / math.sin(self.airstream_angle)
            if distance < 0:
                in_airstream = False
            
            #Work out the power of the airstream
            if in_airstream:
                airstream_size_local = distance * math.tan(self.airstream_spread / 2) + self.airstream_size / 2
                local_power = (self.power + 3) * 300 * (self.airstream_size) / airstream_size_local / airstream_size_local
                if local_power > 70:
                    local_power = 70
                local_power_x = local_power * math.sin(self.airstream_angle)
                local_power_y = local_power * math.cos(self.airstream_angle)
            
            
        return (local_power_x, local_power_y)

class Cauldron:
    def __init__(self):
        self.images = []
        self.image = pygame.image.load("cauldron.png")
        self.images.append(self.image)
        for i in range(1,9):
            self.image = pygame.image.load("cauldron"+str(i)+".png")
            self.images.append(self.image)
        self.rect = pygame.Rect(SCREENSIZE[0]/2-65,SCREENSIZE[1]-103,121,45)
        self.i = 0
    
    def blit(self,screen):
        screen.blit(self.images[self.i],(SCREENSIZE[0]/2-65,SCREENSIZE[1]-300))
        self.i += 1
        if self.i == 9:
            self.i = 0

class Gui:
    def __init__(self):
        self.red = pygame.font.Font(None,16)
        self.orange = pygame.font.Font(None,16)
        self.yellow = pygame.font.Font(None,16)
        self.redimg = pygame.image.load("feather-small.png")
        self.orangeimg = pygame.image.load("feather2-small.png")
        self.yellowimg = pygame.image.load("feather3.png")
        
    def blit(self,screen,feathersleft):
        redtext = self.red.render("x"+str(feathersleft[0]), 1, [255,255,0])
        orangetext = self.orange.render("x"+str(feathersleft[1]), 1, [255,255,0])
        yellowtext = self.yellow.render("x"+str(feathersleft[2]), 1, [255,255,0])
        screen.blit(redtext,(10,5))
        screen.blit(orangetext,(60,5))
        screen.blit(yellowtext,(110,5))
        screen.blit(self.redimg,(15,5))
        screen.blit(self.orangeimg,(65,5))
        screen.blit(self.yellowimg,(115,5))
        

class Game:
    def __init__(self,screen):        
        self.screen = screen
        
        self.background = pygame.image.load("background.png")
        self.mouseimg = pygame.image.load("staff.png")
        pygame.mouse.set_visible(0)
        
        self.B = Bird()
        self.L = Level()
        self.C = Cauldron()
        self.G = Gui()
        
        for i in range(0,7):
            self.L.load_level()
            
            feathersneeded = self.L.feathers_needed
            
            self.featherlist = []
            
            quit = 0
            lost = 0
            returntomenu = 0
            
            Clock = pygame.time.Clock()
            iter = 0
            clicked = 0
            while True:
                Clock.tick(10)
                pygame.event.pump()
                mouse_pos = pygame.mouse.get_pos()
                self.screen.blit(self.background,(0,0))
                self.G.blit(self.screen,feathersneeded)
                self.L.blit(self.screen)
                
                if pygame.key.get_pressed()[K_ESCAPE]:
                    returntomenu = 1
                    break
                
                if feathersneeded[0] == 0 and feathersneeded[1] == 0 and feathersneeded[2] == 0:
                    break
                
                for i in self.featherlist:
                    pos = i.get_pos()
                    if self.C.rect.contains(i.rect) == 1 and i.in_cauldron_var == 0:
                        if i.type == "Red":
                            feathersneeded[0] -= 1
                            if feathersneeded[0] < 0:
                                lost = 1
                        if i.type == "Orange":
                            feathersneeded[1] -= 1
                            if feathersneeded[1] < 0:
                                lost = 1
                        if i.type == "Yellow":
                            feathersneeded[2] -= 1
                            if feathersneeded[2] < 0:
                                lost = 1
                        i.in_cauldron()
                    for j in self.L.machines:
                        wind = j.get_wind(pos)
                        if j.rect.collidepoint(mouse_pos) == 1 and pygame.mouse.get_pressed()[0] == 1 and clicked == 0:
                            clicked = 1
                            j.change_state()
                        i.move(wind)
                    i.blit(self.screen)
                if randint(0,75-iter-self.L.levelno * 5) == 0:
                    feather = self.B.drop_feather()
                    self.featherlist.append(feather)
                    iter = 0
                if pygame.mouse.get_pressed()[0] == 0:
                    clicked = 0
                self.B.move()
                self.C.blit(self.screen)
                self.B.blit(self.screen)
                self.screen.blit(self.mouseimg,(mouse_pos))
                pygame.display.flip()
                iter += 1
                if lost == 1:
                    break
            if lost or returntomenu:
                break
