from random import *
from pygame.locals import *
import pygame
import time
import math

SCREENSIZE = [800,600]

class Windmachine:
    def __init__(self,
                time,
                power,
                base_image,
                pos,
                size,
                airstream_pos,
                airstream_size,
                airstream_angle,
                airstream_spread):
        """time is how long you want it to stay on for, -1 for forever,
        power is how powerful it is on a rate of 1-5, size is the height of
        the base image, airstream_size is the width of the airstream, airstream
        angle is the direction of the airstream measured in degrees clockwise from
        vertically upwards"""
        self.time = time
        self.power = power
        self.base_image = base_image
        self.x, self.y = pos
        self.size = size
        #if self.x < (SCREENSIZE[0]/2)-(self.size[0]/2):
        #    self.direction = 1
        #else:
        #    self.direction = 0
        self.speed = self.power*25
        self.width = 50
        self.spread = 50
        self.state = 0
        self.state_times = []
        
        self.airstream_size = airstream_size
        #Convert the angles into radians
        self.airstream_angle = airstream_angle * math.pi / 180
        self.airstream_spread = airstream_spread * math.pi / 180
        #airstream_angle_1 and _2 are the outer boundaries of the airstream
        #expressed in radians from the vertical
        self.airstream_angle_1 = self.airstream_angle - self.airstream_spread / 2
        self.airstream_angle_2 = self.airstream_angle + self.airstream_spread / 2
        #airstream_pos_x, _y are the central point of the airstream
        self.airstream_pos_x, self.airstream_pos_y = airstream_pos
        #airstream_pos_x_1, etc are the outer points of the airstream
        self.airstream_pos_1_x = self.airstream_pos_x - airstream_size * math.sin(0.5 * math.pi - self.airstream_angle) / 2
        self.airstream_pos_1_y = self.airstream_pos_y + airstream_size * math.cos(0.5 * math.pi - self.airstream_angle) / 2
        self.airstream_pos_2_x = self.airstream_pos_x + airstream_size * math.sin(0.5 * math.pi - self.airstream_angle) / 2
        self.airstream_pos_2_y = self.airstream_pos_y - airstream_size * math.cos(0.5 * math.pi - self.airstream_angle) / 2
        
        print "(", self.airstream_pos_1_x, self.airstream_pos_1_y, ")", self.airstream_angle_1
        print "(", self.airstream_pos_2_x, self.airstream_pos_2_y, ")", self.airstream_angle_2
    
    def blit(self,screen):
        screen.blit(self.base_image,(self.x,self.y))
    
    def change_state(self):
        self.state = 1 - self.state
        self.state_times.append([self.state,time.time()])
    
    def get_wind(self,wind_pos):
        
        local_power_x, local_power_y = (0, 0)
        if self.state == 1:
            wind_pos_x, wind_pos_y = wind_pos
            #Find the points at which the airstream intercepts the vertical
            #line drawn through wind_pos
            tangent = math.tan(self.airstream_angle_1) 
            if tangent != 0:
                intercept_1 = (wind_pos_x - self.airstream_pos_1_x) / tangent + self.airstream_pos_1_y
                distance_1 = (wind_pos_x - self.airstream_pos_1_x) / math.sin(self.airstream_angle_1)
            else:
                intercept_1 = None
                distance_1 = None
            
            tangent = math.tan(self.airstream_angle_2) 
            if tangent != 0:
                intercept_2 = (wind_pos_x - self.airstream_pos_2_x) / math.tan(self.airstream_angle_2) + self.airstream_pos_2_y
                distance_2 = (wind_pos_x - self.airstream_pos_2_x) / math.sin(self.airstream_angle_2)
            else:
                intercept_2 = None
                distance_2 = None
            
            #Various tests to see if we're in the windstream
            upper_limit = 9999999
            if distance_1 > 0 and math.sin(self.airstream_angle_1) > 0:
                upper_limit = intercept_1
            elif distance_2 > 0 and math.sin(self.airstream_angle_2) < 0:
                upper_limit = intercept_2
                
            lower_limit = 0
            if distance_1 > 0 and math.sin(self.airstream_angle_1) < 0:
                lower_limit = intercept_1
            elif distance_2 > 0 and math.sin(self.airstream_angle_2) > 0:
                lower_limit = intercept_2
                
            in_airstream = False
            if wind_pos_y < upper_limit and wind_pos_y > lower_limit:
                in_airstream = True
            if distance_1 < 0 and distance_2 is None:
                in_airstream = False
            if distance_2 < 0 and distance_1 is None:
                in_airstream = False
            
            #Work out the power of the airstream
            if in_airstream:
                distance = (wind_pos_x - self.airstream_pos_x) / math.sin(self.airstream_angle)
                airstream_size_local = distance * math.tan(self.airstream_spread / 2) + self.airstream_size / 2
                print "Local size, size, power:", airstream_size_local, self.airstream_size, self.power
                local_power = 10 * self.power * (self.airstream_size + 10) / airstream_size_local / airstream_size_local
                local_power_x = local_power * distance * math.sin(self.airstream_angle)
                local_power_y = local_power * distance * math.cos(self.airstream_angle)        
            print wind_pos_x, self.airstream_pos_1_x, self.airstream_angle_1
            print "In airstream: ", in_airstream
            print "Intercepts:", intercept_1, intercept_2
            print "Distances:", distance_1, distance_2
            print "Limits:", upper_limit, lower_limit
            
        return (local_power_x, local_power_y)

    
    """base_image, pos, size, airstream_pos, airstream_size, airstream_angle, airstream_spread):"""

a = Windmachine(10,5,'fred',(100,100),15,(0,0),20,45,90)
print a.get_wind((400,800))